package be.cylab.java.wowa.training;

import java.util.logging.Logger;

/**
 * Parameters for the Trainer object.
 */
public class TrainerParameters {

    /**
     * Constant for choosing the selection method. SELECTION_METHOD_RWS for the
     * roulette wheel selection method
     */
    public static final int SELECTION_METHOD_RWS = 374;
    /**
     * Constant for choosing selection method. SELECTION_METHOD_TOS for the
     * tournament selection
     */
    public static final int SELECTION_METHOD_TOS = 942;
    /**
     * Constant for choosing generation method.
     * POPULATION_INITIALIZATION_RANDOM for random initialization
     */
    public static final int POPULATION_INITIALIZATION_RANDOM = 758;
    /**
     * Constant for choosing generation method.
     * POPULATION_INITIALIZATION_QUASI_RANDOM
     * for quasi_random initialization
     */
    public static final int POPULATION_INITIALIZATION_QUASI_RANDOM = 329;

    private Logger logger;
    private int population_size;
    private int crossover_rate;
    private int mutation_rate;
    private int selection_method;
    private int number_of_parents;
    private int population_initialization_method;

    private int max_generation_number;
    //private float trigger_distance;

    /**
     * Contains parameters for the WOWA trainer.
     *
     * @param logger
     * @param population_size Size of the population for the Genetic Algorithm
     * (greater than 2)
     * @param crossover_rate Crossover rate for the genetic algorithm.
     * Percentage of the population kept to generate the next generation (1-99)
     * @param mutation_rate Mutation rate for the genetic algorithm. percentage
     * of genes mutated in the next generation (1-99)
     * @param selection_method Method to select parents for the next generation.
     * RWS or TOS
     * @param max_generation_number Max generation number before the algorithm
     * stop
     */
    public TrainerParameters(
            final Logger logger,
            final int population_size,
            final int crossover_rate,
            final int mutation_rate,
            final int max_generation_number,
            final int selection_method,
            final int population_initialization_method
            ) {

        this.setLogger(logger);
        this.setPopulationSize(population_size);
        this.setCrossoverRate(crossover_rate);
        this.mutation_rate = mutation_rate;
        this.setSelectionMethod(selection_method);
        this.setMaxGenerationNumber(max_generation_number);
        this.setPopulationInitializationMethod(
                population_initialization_method);

    }

    /**
     * Getter for number of parents.
     *
     * @return int
     */
    final int getNumberOfParents() {
        return this.number_of_parents;
    }

    /**
     * @return Logger
     */
    final Logger getLogger() {
        return logger;
    }

    private void setLogger(final Logger logger) {
        this.logger = logger;
    }

    /**
     * Getter for population_size.
     *
     * @return int
     */
    final int getPopulationSize() {
        return this.population_size;
    }

    /**
     * Setter population_size.
     *
     * @param population_size, must be greater than 2
     * @throws ArithmeticException Population must be higher than 2
     */
    private void setPopulationSize(final int population_size)
            throws ArithmeticException {
        if (population_size < 2) {
            throw new ArithmeticException("Population must be higher than 2");
        }
        this.population_size = population_size;
    }

    /**
     * Setter for crossover rate.
     *
     * @param crossover_rate Determine the number of parent selected for the
     * next generation. Value between 1-99
     */
    private void setCrossoverRate(final int crossover_rate) {
        this.crossover_rate = crossover_rate;
        int nbr_parents = Math.round(this.population_size
                * (1 - (float) crossover_rate / 100));
        if (nbr_parents % 2 == 1) {
            nbr_parents++;
        }
        this.number_of_parents = nbr_parents;
    }

    /**
     * Getter for mutation rate.
     *
     * @return int
     */
    final int getMutationRate() {
        return mutation_rate;
    }

    /**
     * Getter for selection method.
     *
     * @return int
     */
    final int getSelectionMethod() {
        return selection_method;
    }

    /**
     * Setter for selection method.
     *
     * @param selection_method int, must be RWS or TOS method
     * @throws IllegalArgumentException Value must be SELECTION_METHOD_RWS or
     * SELECTION_METHOD_TOS
     */
    private void setSelectionMethod(final int selection_method) {
        if (selection_method != SELECTION_METHOD_RWS
                && selection_method != SELECTION_METHOD_TOS) {
            throw new IllegalArgumentException("Not a valid selection method");
        }
        this.selection_method = selection_method;
    }

    /**
     * Getter for max generation number.
     *
     * @return int
     */
    final int getMaxGenerationNumber() {
        return max_generation_number;
    }

    /**
     * Setter for max generation number.
     *
     * @param max_generation_number number of generation before algorithm stop
     */
    private void setMaxGenerationNumber(final int max_generation_number) {
        this.max_generation_number = max_generation_number;
    }

    /**
     * Setter for initialization method.
     * @param population_initialization_method
     */
    final void setPopulationInitializationMethod(
            final int population_initialization_method) {
        if (population_initialization_method != POPULATION_INITIALIZATION_RANDOM
                && population_initialization_method
                != POPULATION_INITIALIZATION_QUASI_RANDOM) {
            throw new IllegalArgumentException(
                    "Incorrect initialization method");
        }
        this.population_initialization_method =
                population_initialization_method;
    }

    /**
     * @return
     */
    final int getPopulationInitializationMethod() {
        return this.population_initialization_method;
    }
}
